part of '../flutterx_live_data.dart';

abstract class LiveData<T> {
  final Map<Observer<T>, ObserverWrapper<T>> _observers = {};
  late T _data;
  int _version = _uninitialized;
  int _activeCount = 0;

  int get version => _version;

  bool get isActive => _activeCount > 0;

  bool get initialized => _version > _uninitialized;

  T get value => _data;

  @protected
  void onActive() {}

  @protected
  void onInactive() {}

  @protected
  bool shouldUpdateValue(T oldValue, T newValue) => newValue != oldValue;

  @protected
  void setData(T value) {
    if (!initialized || shouldUpdateValue(_data, value)) {
      _data = value;
      invalidate();
    }
  }

  void invalidate() {
    _version++;
    _dispatchingValue(null);
  }

  void observeForever(Observer<T> onChanged, {Dispatcher? dispatcher}) =>
      addObserverWrapper(ObserverWrapper<T>(this, onChanged, dispatcher));

  void removeObserver(Observer<T> observer) => _observers.remove(observer)?.onRemove();

  void addObserverWrapper(ObserverWrapper<T> observer) {
    assert(!_observers.containsKey(observer.onChanged), 'observer already registered');
    _observers[observer.onChanged] = observer;
    observer.onAdded();
  }

  void removeObserverWrapper(ObserverWrapper<T> observer) => removeObserver(observer.onChanged);

  void _changeActiveState(ObserverWrapper<T> observer, {required bool active}) {
    if (observer.active == active) return;
    if (observer.active = active) {
      _activeCount++;
      if (_activeCount == 1) onActive();
    } else {
      _activeCount--;
      if (_activeCount == 0) onInactive();
    }
    if (active) _dispatchingValue(observer);
  }

  void _dispatchingValue(ObserverWrapper<T>? initiator) {
    if (initialized) {
      if (initiator != null) {
        _considerNotify(initiator);
      } else {
        _observers.values.toList(growable: false).forEach(_considerNotify);
      }
    }
  }

  void _considerNotify(ObserverWrapper<T> observer) {
    if (observer.active && _version > observer.lastVersion) {
      observer.lastVersion = _version;
      observer._dispatch(_data);
    }
  }

  @override
  String toString() => '$runtimeType${initialized ? (identical(value, this) ? '' : '($value)') : '[uninitialized]'})';
}

class ObserverWrapper<T> {
  final LiveData<T> data;

  final Observer<T> onChanged;

  final Dispatcher? dispatcher;

  bool active = false;

  int lastVersion = _uninitialized;

  ObserverWrapper(this.data, this.onChanged, this.dispatcher);

  void changeActiveState({required bool active}) => data._changeActiveState(this, active: active);

  void onAdded() => changeActiveState(active: true);

  void onRemove() => changeActiveState(active: false);

  late T _pendingData;
  bool _dispatchingPendingData = false;

  void _dispatch(T data) {
    final dispatcher = this.dispatcher;
    if (dispatcher == null) return onChanged(data);
    _pendingData = data;
    if (_dispatchingPendingData) return;
    _dispatchingPendingData = true;
    dispatcher.post(() {
      onChanged(_pendingData);
      _dispatchingPendingData = false;
    });
  }
}

const int _uninitialized = -1;

typedef Observer<T> = void Function(T value);

class MediatorLiveData<T> extends LiveData<T> with MediatorMixin<T> {}
